"""Settings to access not public era5 data."""

from typing import Tuple


def ifs_settings(sampling="daily", ifs_data_path=None, ifs_file_names=None) -> Tuple[str, str]:
    """
    Check for sampling as only hourly resolution is supported by ifs and return path on HPC systems.

    :param sampling: temporal resolution to load data for, only hourly supported (default "daily")

    :return: HPC path
    """
    if sampling == "hourly":  # pragma: no branch
        IFS_DATA_PATH = ifs_data_path or "."
        FILE_NAMES = ifs_file_names or "*.nc"
    else:
        raise NameError(f"Given sampling {sampling} is not supported, only hourly sampling can be used.")
    return IFS_DATA_PATH, FILE_NAMES
