from mlair.configuration.join_settings import join_settings
from mlair.helpers.data_sources.data_loader import get_data, correct_stat_name, create_url


class TestGetData:

    def test(self):
        opts = {"base": join_settings()[0], "service": "series", "station_id": 'DEBW107', "network_name": "UBA",
                "parameter_name": "o3,no2"}
        assert get_data(opts, headers={}) == [[17057, 'UBA', 'DEBW107', 'O3'], [17058, 'UBA', 'DEBW107', 'NO2']]


class TestCreateUrl:

    def test_minimal_args_given(self):
        url = create_url("www.base.edu", "testingservice")
        assert url == "www.base.edu/testingservice/"

    def test_given_kwargs(self):
        url = create_url("www.base2.edu/", "testingservice", mood="happy", confidence=0.98)
        assert url == "www.base2.edu/testingservice/?mood=happy&confidence=0.98"

    def test_single_kwargs(self):
        url = create_url("www.base2.edu/", "testingservice", mood="undefined")
        assert url == "www.base2.edu/testingservice/?mood=undefined"

    def test_none_kwargs(self):
        url = create_url("www.base2.edu/", "testingservice", mood="sad", happiness=None, stress_factor=100)
        assert url == "www.base2.edu/testingservice/?mood=sad&stress_factor=100"

    def test_param_id(self):
        url = create_url("www.base.edu", "testingservice", param_id="2001")
        assert url == "www.base.edu/testingservice/2001"

    def test_param_id_kwargs(self):
        url = create_url("www.base.edu", "testingservice", param_id=2001, mood="sad", happiness=None, stress_factor=100)
        assert url == "www.base.edu/testingservice/2001?mood=sad&stress_factor=100"

        url = create_url("www.base.edu", "testingservice", param_id=2001, mood="sad", series_id=222)
        assert url == "www.base.edu/testingservice/2001?mood=sad&series_id=222"


class TestCorrectStatName:

    def test_nothing_to_do(self):
        assert correct_stat_name("dma8eu") == "dma8eu"
        assert correct_stat_name("max") == "max"

    def test_correct_string(self):
        assert correct_stat_name("maximum") == "max"
        assert correct_stat_name("minimum") == "min"
        assert correct_stat_name("average_values") == "mean"

